MainProgram: 
  ErrorAmount = 12 // set % error you want the simulator to have
  REAL = false // Test Simulator, set to true to test a real robot 
  // note: real robots typically must be RROS compatible 
  //       RROS = RobotBASIC Robot Operating System (see our webpage)
  // note: Depending on the physical robot, some changes may have to be made for accuracy 
  gosub Introduction
  gosub CreateTestEnvironment
  gosub InitRobot
  gosub Setup
  gosub LineSensors  
  gosub WheelCounters
  gosub RangeSensors 
  gosub CompassTest
  gosub BeaconTest  
end
  
Introduction:
  ClearScr  
  if REAL     
    print "This program will now test the RB-9 robot (or other compatible robot)."
  else      
    print "This program will now test the RobotBASIC simulated robot." 
    delay 2000
  endif 
return

CreateTestEnvironment:
  if REAL
    print
    print "You need a line on the floor (or posterboard), perpendicular to a wall."
    print "The line should be wide enough to cover all 3 of the robot's line sensors."
    print "A second line should be perpendicular to the first."
    print "A beacon should be set at the right end of the second line."
    print "Test the simulator to see the setup."
    delay 5000
    ClearScr
  else
    ClearScr 
    SetColor BLUE
    LineWidth 5                      
    Line 300,3,500,3  
    SetColor GREEN
    LineWidth 8                 
    Line 400,10,400,90
    Line 350,50,450,50  
    SetColor Black
    r=5
    circle 625-r,50-r,625+r,50+r
    xyString 640,42,"beacon"
  endif 
  AddButton " LEFT ", 610,200
  AddButton " RIGHT ", 690,200 
  AddButton " FORWARD ", 635,165  
  AddButton " REVERSE ", 635,235
  AddButton " DONE ",     648,290
  LineWidth 1 
  xyString 80,30,"CURRENT TEST"
  Rectangle 45,50,220,80 
  LineWidth 1
  Line 0,320,799,320
  Rectangle 215,150,585,320 
  xyString 290,128,"INSTRUCTIONS/INFORMATION"
return

InitRobot:
  if REAL
    // perform appropriate initialization here for your real robot
    // download the RROS manual from RobotBASIC.org for more details
    rLocate 400,300  
  else
    rLocate 400,50
    rInvisible GREEN,BLACK 
    rSpeed 20
    rSlip ErrorAmount    // movement and sensor errors based on ErrorAmount
  endif
  SetColor Black
  xyString 20,175,"SENSOR READINGS" 
  Rectangle 0,195,215,320
  xyString 5,200,"Line Sensors = "
  xyString 5,220,"Front Range  = "
  xyString 5,240,"Left  Range  = "
  xyString 5,260,"Right Range  = "
  xyString 5,280,"Comp Heading = "
  xyString 5,300,"Beacon found = "
  gosub ShowSensors
return

ShowSensors:  
  xyString 150,200,rSense(),   "  " 
  r = rRange()
  if r<60
    xyString 150,220,r,"  " 
  else
    xyString 150,220,"-  "
  endif
  r = rRange(-90)
  if r <60
    xyString 150,240,r,"  "
  else
    xyString 150,240,"-  "
  endif
  r = rRange(+90)
  if r<60
    xyString 150,260,r,"  " 
  else
    xyString 150,260,"-  "
  endif
  c = rCompass()
  xyString 150,280,c, "    "
  if not REAL
    if rBeacon(BLACK) and c>80 and c<100 // avoid false readings on simulator
      xyString 150,300,"TRUE "
    else
      xyString 150,300,"FALSE"
    endif
  else
    if rBeacon(0) // use your beacon number if desired
      xyString 150,300,"TRUE "
    else
      xyString 150,300,"FALSE"
    endif
  endif 
return

Setup:  
  // note: The manual setup using buttons was not mentioned in the article
  //       it was included here to show how a real robot could be moved
  //       using the buttons provided (they will also move the simulator)
  Rectangle 45,50,220,80
  xyString 100,57,"Setup"          
  Rectangle 215,150,585,320
  xyString 220,160,"Use buttons on right to ensure robot is"
  xyString 220,180,"centered on lines with all three line"
  xyString 220,200,"sensors on the vertial line."
  xyString 220,230,"Note: Sensor readings are valid during"
  xyString 220,250,"the moving process."
  xyString 220,285,"Click DONE when finished."   
  repeat
    Button = LastButton()
    if Button = " FORWARD " then rForward 1
    if Button = " REVERSE " then rForward -1
    if Button = " LEFT "    then rTurn -1
    if Button = " RIGHT "   then rTurn 1
    if Button <> "" then gosub ShowSensors
  until Button = " DONE "
  AddButton " PERFORM TEST ",75,90
  AddButton " FULL AUTO ",90,120
  RemoveButton " LEFT "
  RemoveButton " RIGHT "
  RemoveButton " FORWARD " 
  RemoveButton " REVERSE "
  RemoveButton " DONE "  
return 

LineSensors:
  Rectangle 45,50,220,80                            
  xyString 80,57,"LineSensors"          
  Rectangle 215,150,585,320
  xyString 220,160,"The robot will rotate to see if the"
  xyString 220,180,"line sensors work as expected"  
  xyString 220,210,"Use PERFORM TEST to run CURRENT test."
  xyString 220,230,"Use FULL AUTO to run ALL tests."
  Action = 0
  repeat
    Button = LastButton() 
    if Button = " PERFORM TEST " then Action = 1
    if Button = " FULL AUTO " then Action = 2
  until Action 
  gosub ShowSensors
  xyString 350,335,"TEST RESULTS"
  if rSense() <> 7
    xyString 5,355,"The line sensors should be all on (assuming you setup the robot properly)."
    xyString 5,375,"They are reporting a ",rSense()," so they have failed."
    xyString 5,395,"Fix the problem and rerun the test."
    gosub StopTesting
  endif
  // test other line sensor states
  rTurn -45
  gosub ShowSensors
  if rSense()=0
    xyString 5,355,"The line sensors seem to be working properly."
    xyString 5,375,"All sensors will turn on and off."
  else
    xyString 5,355,"If the robot has turned away from the line, the line sensors are not"
    xyString 5,375,"working properly. Currently they should be all off, but they read ",rSense(),"." 
    xyString 5,400,"Fix the line sensors and rerun this test program."
    xyString 5,420,"NOTE: If the robot has not moved away from the line, then the motors"
    xyString 5,440,"      must be fixed before the tests can continue."
    gosub StopTesting
  endif
  rTurn 45
  gosub ShowSensors
  // now move robot back to setup position 
  if rSense() = 0 // not even close to line
    xyString 5,395,"The robot however, could not find the line again. Perhaps there"
    xyString 5,415,"is a problem with the encoders. The robot must be able to move"                            
    xyString 5,435,"reasonably accurately to run the tests."
    xyString 5,455,"Fix the problem and rerun this test program."
    gosub StopTesting
  endif    
  // some of the line sensors are on the line
  // make tiny movements to get all sensors back on the line
  LT = 1 // any init will do here - as robot must be on the line at this point
  num = 0 // count the number of tries it takes to center on the line again
  for i=1 to 20
    if rSense() = 0 then rTurn LT      // reverse last movement if line lost
    if rSense() = 1 then rTurn 1\LT=-1
    if rSense() = 3 then rTurn 1\LT=-1
    if rSense() = 6 then rTurn -1\LT=1
    if rSense() = 4 then rTurn -1\LT=1 
    gosub ShowSensors 
    num++ 
    if rSense()=7 then break
  next
  // check ability to center on the line again 
  if num>5 // problem indicated if more than 5 tries are needed (adjust a preferred)
    xyString 5,395,"The robot, however, took too long to find the line again."
    xyString 5,415,"Perhaps the motors are too fast or too slow."                            
    xyString 5,435,"The robot must be able to move reasonably" 
    xyString 5,455,"accurately to run the test. "
    xyString 5,475,"Fix the problem and rerun this test program." 
    gosub StopTesting 
  else
    xyString 295,375," and they were used to find the line again."
  endif                                                   
return 

StopTesting:
  end
return   
                

WheelCounters:  
  Rectangle 45,50,220,80                            
  xyString 80,57,"Wheel Counters"          
  Rectangle 215,150,585,320
  xyString 220,160,"The robot will rotate 90 degrees"
  xyString 220,180,"right, expecting to find the line" 
  xyString 220,200,"to check for accuracy.  It will"
  xyString 220,220,"then rotate left 90 degrees to"
  xyString 220,240,"check repeatability."             
  if Action = 1 then Action = 0
  while not Action   
    Button = LastButton()
    if Button = " PERFORM TEST " then Action = 1
    if Button = " FULL AUTO " then Action = 2
  wend  
  rTurn 90
  gosub ShowSensors
  s = rSense()
  if s=7 
    xyString 5,400,"Wheels appear to turn accurately."
  elseif s=6
    xyString 5,400,"Wheels appear to turn slightly too far." 
  elseif s=4
    xyString 5,400,"Wheels appear to turn too far."        
  elseif s=3
    xyString 5,400,"Wheels are not turning quite far enough."
  elseif s=1
    xyString 5,400,"Wheels are not turning enough."                                     
  elseif s=0
    xyString 5,400,"Wheel counters need major calibration."  
    xyString 5,420,"Correct the problem and rerun the test."
    gosub StopTesting
  endif
  rTurn -90
  gosub ShowSensors
  s=rSense()
  if (s=7) or (s=6) or (s=3)
    xyString 5,420,"Counters seem reasonably accurate for repeatability."
  elseif (s=4) or (s=1)
    xyString 5,420,"Counters have problems with repeatability"
  else
    xyString 5,420,"Counters also have significant repeatability problems."
    xyString 5,450,"Correct problems and rerun this test."
    gosub StopTesting
  endif
  // now move robot back to setup position to prepare for next test
  // robot should be at least partically on the line
  repeat
    if rSense() = 0 then rTurn -LT // get back to line
    if rSense() = 1 then rTurn 1\LT=1
    if rSense() = 3 then rTurn 1\LT=1
    if rSense() = 6 then rTurn -1\LT=-1
    if rSense() = 4 then rTurn -1\LT=-1   
    gosub ShowSensors  
  until rSense()=7
return

RangeSensors:   
  // note: The ranging turret is mounted in the front center of the robot
  //       this means that reading to the left and right are 20 pixels
  //       longer than a front reading (the radius of the simulated robot)
  //       notice the correction for this below
  Rectangle 45,50,220,80                            
  xyString 80,57,"Range Sensors"          
  Rectangle 215,150,585,320
  xyString 220,160,"The robot will rotate to see if the"
  xyString 220,180,"range sensors measure approximately the"
  xyString 220,200,"same distance from the wall. Note: Only"
  xyString 220,220,"three sensor positions are tested."
  if Action = 1 then Action = 0
  while not Action   
    Button = LastButton()
    if Button = " PERFORM TEST " then Action = 1
    if Button = " FULL AUTO " then Action = 2
  wend
  Dist1 = rRange()
  rTurn 90
  Dist2 = rRange(-90) 
  gosub ShowSensors 
  if Dist1=Dist2
    xyString 5,440,"The left range sensor is correct."
  elseif abs(Dist1-(Dist2-19))<3  // 20 added due to position of turret
    xyString 5,440,"The left range sensor is acceptable." 
  else
    xyString 5,440,"The left range sensor is inaccurate."
  endif                                               
  rTurn -180
  Dist3 = rRange(90) 
  gosub ShowSensors 
  if Dist1=Dist3
    xyString 5,460,"The right range sensor is correct."
  elseif abs(Dist1-(Dist2-19))<3  // 20 added due to position of turret
    xyString 5,460,"The right range sensor is acceptable." 
  else
    xyString 5,460,"The right range sensor is inaccurate."
  endif 
  rturn 90 // return to original position
  gosub ShowSensors
  repeat
    if rSense() = 0 then rTurn -LT // get back to line 
    if rSense() = 1 then rTurn 1\LT=1
    if rSense() = 3 then rTurn 1\LT=1
    if rSense() = 6 then rTurn -1\LT=-1
    if rSense() = 4 then rTurn -1\LT=-1   
    gosub ShowSensors  
  until rSense()=7 
return

CompassTest:
  Rectangle 45,50,220,80                            
  xyString 80,57,"Compass Test"          
  Rectangle 215,150,585,320
  xyString 220,160,"The robot will rotate to see if"
  xyString 220,180,"the angles moved match the"
  xyString 220,200,"compass readings. Note: Only"
  xyString 220,220,"two positions are tested."
  if Action = 1 then Action = 0
  while not Action   
    Button = LastButton()  
    if Button = " PERFORM TEST " then Action = 1
    if Button = " FULL AUTO " then Action = 2
  wend
  heading = rCompass()  
  rTurn 90
  gosub ShowSensors
  h90 = heading+90
  if h90>359 then h90-=360    
  if abs(rCompass()-h90)<=3
    xyString 5,480,"First compass reading is acceptable."
  else
    xyString 5,480,"First compass reading is out of range."
  endif  
  rTurn -90
  gosub ShowSensors
  heading = rCompass()
  rTurn -90
  gosub ShowSensors
  h90 = heading-90
  if h90<0 then h90+=360   
  if abs(rCompass()-h90)<=3
    xyString 5,500,"Second compass reading is acceptable."
  else
    xyString 5,500,"Second compass reading is out of range."
  endif
  rTurn 90 // return to original position
  gosub ShowSensors
  if not rSense() // not close to original position
    // assume error has occurred
    xyString 5,520,"Error: Robot has lost its bearings."      
    xyString 5,540,"Program has been terminated."
  endif
  repeat
    if rSense() = 0 then rTurn -LT // get back to line
    if rSense() = 1 then rTurn 1\LT=1
    if rSense() = 3 then rTurn 1\LT=1
    if rSense() = 6 then rTurn -1\LT=-1
    if rSense() = 4 then rTurn -1\LT=-1   
    gosub ShowSensors  
  until rSense()=7   
return

BeaconTest:
  Rectangle 45,50,220,80                            
  xyString 80,57,"Beacon Test"          
  Rectangle 215,150,585,320
  xyString 220,160,"The robot will rotate to where"
  xyString 220,180,"a beacon should be located."
  xyString 220,200,"A check is then made to see if"
  xyString 220,220,"the beacon is detected."
  if Action = 1 then Action = 0
  while not Action   
    Button = LastButton()  
    if Button = " PERFORM TEST " then Action = 1
    if Button = " FULL AUTO " then Action = 2
  wend
  rTurn 90
  gosub ShowSensors
  b = rBeacon(0) // checks for color BLACK for simulator or ANY beacon for the REAL robot
  if not b then b=rBeacon(0)  // check twice to improve reliability for SLIP errors
  if b 
    xyString 5,520,"The Beacon was found as expected."
  else
    xyString 5,520,"Error: No beacon detected."
  endif
  rTurn -90 // return to setup position
  gosub ShowSensors
  repeat
    if rSense() = 0 then rTurn -LT // get back to line
    if rSense() = 1 then rTurn 1\LT=1
    if rSense() = 3 then rTurn 1\LT=1
    if rSense() = 6 then rTurn -1\LT=-1
    if rSense() = 4 then rTurn -1\LT=-1   
    gosub ShowSensors  
  until rSense()=7 
  xyString 5,540,"Testing Complete"
return
  

